/*
 *	Ohio Trollius
 *	Copyright 1996 The Ohio State University
 *	RBD
 *
 *	$Id: getcount.c,v 6.1 96/11/23 22:51:57 nevin Rel $
 *
 *	Function:	- get # top-level elements
 *	Accepts:	- ptr to status
 *			- datatype
 *			- ptr to count
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>

int
MPI_Get_count(pstat, dtype, pcount)

MPI_Status		*pstat;
MPI_Datatype		dtype;
int			*pcount;

{
	lam_initerr();
	lam_setfunc(BLKMPIGETCOUNT);
/*
 * Check the arguments.
 */
	if ((pcount == 0) || (pstat == 0)) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPIGETCOUNT, lam_mkerr(MPI_ERR_ARG, 0)));
	}

	if ((dtype == MPI_DATATYPE_NULL) || ((dtype != pstat->st_dtype) &&
				(pstat->st_dtype != MPI_DATATYPE_NULL) &&
				(pstat->st_dtype != MPI_PACKED) &&
				(dtype != MPI_PACKED))) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPIGETCOUNT, lam_mkerr(MPI_ERR_TYPE, 0)));
	}
/*
 * If different datatypes, recompute count.
 */
	if (dtype != pstat->st_dtype) pstat->st_count = -1;

	pstat->st_dtype = dtype;
/*
 * If the count is not available, compute it.
 */
	if (pstat->st_count == -1) {
		lam_docount(pstat->st_length, dtype,
				&(pstat->st_count), &(pstat->st_nelem));
	}

	*pcount = pstat->st_count;

	lam_resetfunc(BLKMPIGETCOUNT);
	return(MPI_SUCCESS);
}
